part of jaguar_reflect.base;

class InstanceWrapper implements Wrapper {
  final j.RouteWrapper _routeWrapper;

  String get id => _routeWrapper.id;

  final Type interceptorType;

  final _Pre pre;

  final _Post post;

  InstanceWrapper(
      this._routeWrapper, this.pre, this.post, this.interceptorType);

  j.Interceptor createInterceptor(j.ContextImpl ctx, j.Response response) =>
      _routeWrapper.createInterceptor();

  FutureOr<j.RouteWrapper> getWrapper(j.Context ctx, j.Response response) =>
      _routeWrapper;

  factory InstanceWrapper.build(j.RouteWrapper wrapper) {
    ClassMirror icm = _getInterceptorMirror(wrapper);
    _Pre pre;
    _Post post;
    DeclarationMirror preM = icm.declarations[#pre];
    if (preM is MethodMirror) {
      pre = new _Pre.build(preM);
    }
    DeclarationMirror postM = icm.declarations[#post];
    if (postM is MethodMirror) {
      post = new _Post.build(postM);
    }

    return new InstanceWrapper(wrapper, pre, post, icm.reflectedType);
  }

  static ClassMirror _getInterceptorMirror(final j.RouteWrapper wrapper) {
    final ClassMirror cm = reflectClass(wrapper.runtimeType);
    final ClassMirror wcm = cm.superclass;

    {
      final type1 = wcm.qualifiedName;
      final type2 = reflectType(j.RouteWrapper).qualifiedName;
      if (type1 != type2) {
        throw new Exception('Wrapper must be a subclass of RouteWrapper!');
      }
    }

    if (wcm.typeArguments.length != 1) {
      throw new Exception(
          'Interceptor type must be specified in RouteWrapper type argument!');
    }

    return wcm.typeArguments.first;
  }
}
